#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <pthread.h>

#include <wiimote_api.h>

#include "serie.h"

#define PERIODE_REFRESH     10
#define MOYENNE_SIZE        60
#define MAX                 40

void recal(wiimote_t * parWi);
int ref[3] = {0,0,0};		//valeurs de référence 'Zéro' pour la suite. 

/**
* recal : recalibrate the ZERO
* @param wiimote_t * parWi : wii id.
* @return none
*/
void recal(wiimote_t * parWi)
{
    int k;

    ref[0] = 0;
    ref[1] = 0;
    ref[2] = 0;

    for(k=MOYENNE_SIZE;k;k--)		//effectue MOYENNE_SIZE relevés des valeurs des axes x,y,z
    {
        if(wiimote_is_open(parWi))
        {
            wiimote_update(parWi);
            ref[0] += parWi->axis.x;
            ref[1] += parWi->axis.y;
            ref[2] += parWi->axis.z;
            printf("#");
            usleep(100);
        }
        else
        {
            return;
        }
    }

    printf("\n");

    ref[0] = ref[0]/(MOYENNE_SIZE-1);		//calcul une moyenne. et sauve dans les références.
    ref[1] = ref[1]/(MOYENNE_SIZE-1);
    ref[2] = ref[2]/(MOYENNE_SIZE-1);

    printf("final ref : x = %d y = %d z = %d\n",ref[0], ref[1], ref[2]);	//affiche.
}


/**
* wiiControl : controle le robot ac les infos wii
* @param parPtr : name of the theard, could be everything.
* @return should never return...
*/
 void wiiControl(void * parPtr)
 {
	unsigned char exit[4] = {'m',0,0,3};		//trame envoyée pour quitter le mode pilotage
	unsigned char cmd[3] = {'m',0,0};		//trame envoyée : m, gauche, droite
	volatile int control = 1;			//active / désactive le controle du robot ac la manette.
	signed int gauche = 0, droite = 0, vitesse = 0, tourne = 0;	//vars de travail.
	int timerSoft = 0;		//pour éviter de surcharger le robot ac trop de commande. 
					//Note, la manette semble nécessiter un rafraichissement fréquent

	//Affiche quelques infos du robot (verifie que la comm est bonne.
	jamesWriteString("help\r\n");
	sleep(1);
	//passe en mode pilotage du robot
	jamesWriteString("pilot\r\n");
	sleep(1);

	/*--- Init la wiimote --*/
	wiimote_t wi = WIIMOTE_INIT;
	wiimote_connect(&wi, "00:1F:C5:5C:50:3B");
	wi.mode.acc = 1;		// enable accelerometer
	recal(&wi);

	while (wiimote_is_open(&wi))
	{
		wiimote_update(&wi); 	// synchronize with wiimote

		if(timerSoft == 0)	//Timer Soft pour ne pas saturer le robot
		{
			/*--- boutton A pour suspendre le pilotage. ---*/
			if( wi.keys.a )		
			{
				if(control)
				{
					control = 0;
					printf("suspend\n");
				}
				else
				{
					control = 1;
					printf("activate\n");
				}
			}

			/*--- Calcul des commandes ---*/

			tourne =  wi.axis.x-ref[0];	//recallage des valeurs 
			vitesse = wi.axis.y-ref[1];	//avec les références 

			if(control)	//si le pilotage est actif
			{
				gauche = vitesse + tourne;		//calcul des vitesses moteurs Gauche
				droite = vitesse - tourne;		// et droite
			}
			else		//si le pilotage est désactivé
			{
				gauche = 0;		//les deux consignes sont nulles
				droite = 0;
			}

			if (gauche > MAX) gauche = MAX;			//Borne les valeurs
			if (gauche < -MAX) gauche = -MAX;
			if (droite > MAX) droite = MAX;
			if (droite < -MAX) droite = -MAX;

			/*--- Envois de la consigne ---*/
			cmd[0]='m';		//génère la trame
			cmd[1] = gauche;
			cmd[2] = droite;

			printf("gauche=%4d\tdroite=%4d\n", gauche, droite);	//affiche la consigne
			jamesWriteCmd(cmd, 3);					//envois la commande.
			timerSoft = PERIODE_REFRESH;				//recharge le timer 
		}
		timerSoft--;
		/* micro tempo : ne doit pas etre trop grande */    
		usleep(1000);
	}

	return;
}



int main()
{

	void * wThreadRet;
	pthread_t wThWii, wThSerie;
	
	/* Ouvre le port Série */
	jamesOpenPort();
	
	/* Crée les thread */	
	if( pthread_create( &wThSerie, NULL, jamesRead, (void*)"James serial" ) < 0 )
	{
		printf("Fail to create Thread : James serial \n");
		exit(-1);
	}

	if( pthread_create( &wThWii, NULL, wiiControl, (void*)"Wii Handler" ) < 0 )
	{
		printf("Fail to create Thread : Wii Handler\n");
		exit(-1);
	}

	/*Rejoint les threads */
	(void)pthread_join( wThSerie, &wThreadRet );
	(void)pthread_join( wThWii, &wThreadRet );

    	return 0;
}



