#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <termios.h>
#include <time.h>
#include <curses.h>
#include <sys/types.h>

#include "serie.h"

/******************************************************************\
* PRIVATE SYMBOLIC CONSTANT
\******************************************************************/
#define READ_TIMEOUT   			3            	//time out en seconde.
#define NB_TRY_MAX      		3            	//

//#define DEBUG_SERIAL

/******************************************************************\
* PRIVATE
\******************************************************************/
int mjamesSerialFile;
void (*jamesHandler)(char *) = NULL;

/******************************************************************\
* PRIVATE FUNCTION PROTOTYPES
\******************************************************************/
/**
* jamesWriteString : Send a string throught the serial port.
* @param parPtrString : string to be sent.
* @return wn : number of byte written.
*/
int jamesWriteString( char * parPtrString);

/******************************************************************\
* PRIVATE FUNCTION CODE
\******************************************************************/
/**
* jamesWriteString : Send a string throught the serial port.
* @param parPtrString : string to be sent.
* @return wn : number of byte written.
*/
int jamesWriteString( char * parPtrString)
{
        int wn;
        wn = write(mjamesSerialFile, parPtrString, strlen(parPtrString));
        if(wn < 0)
		printf("SERIAL> String write error : %s \n", parPtrString);
	else
		printf("SERIAL> String Write successfully : %s", parPtrString);
        return (wn);
}
/******************************************************************\
* PUBLIC FUNCTION CODE
\******************************************************************/

/**
* jamesOpenPort : open the serial port describe by SERIAL_PORT
* @param none
* @return mjamesSerialFile : -1 if fail to open.
*/
int jamesOpenPort(void)
{
        struct termios wOptions;     //wOptions of the serial port. will be modified and applies

        /* try to open the serial port (File descriptor mjamesSerialFile) --*/
        mjamesSerialFile = open(JAMES_SERIAL_PORT, O_RDWR | O_NOCTTY | O_NDELAY);
        if(mjamesSerialFile == -1)       //in case of an echec : the port should have been opened.
        {
            fprintf( stderr, "openPort : unable to open %s\n",JAMES_SERIAL_PORT);
            exit(-1);
        }

        /*------ make the File descriptor asynchronous ------*/
        fcntl(mjamesSerialFile, F_SETFL, 0);

        /*------ COnfigure the serial port -----------------*/
        wOptions.c_cflag = james_SERIAL_BAUDRATE | CRTSCTS | CS8 | CLOCAL | CREAD;   //8N1
        wOptions.c_iflag = IGNPAR | IGNCR;
        wOptions.c_oflag = 0;
        wOptions.c_lflag = 0;      //canonic mode : wait for a NL  (LF) to complete.
        wOptions.c_cc[VMIN] = 1;
        wOptions.c_cc[VTIME] = 0;

        tcflush(mjamesSerialFile, TCIFLUSH);

        /*----- apply the settings -----*/
        tcsetattr(mjamesSerialFile, TCSANOW, &wOptions);

        return(mjamesSerialFile);
}

/**
* jamesWriteString : Send a cmd
* @param parCmd : byte array
* @param parLength : longueur
* @return wn : number of byte written.
*/
int jamesWriteCmd(unsigned char * parCmd, int parLength)
{
        int wn;
        wn = write(mjamesSerialFile, parCmd, parLength );
        if(wn < 0)
		printf("SERIAL> write error \n");
	else
		printf("SERIAL> Write successfully ");
        return (wn);
}


/**
* jamesReadAt : poll the serial port and display
* @param parPtr : name of the theard, could be everything.
* @return should never return...
*/
void * jamesRead(void * parPtr)
{
	int wNBytes;         //number of bytes read
	char * ptrTemp;
	char bfrTemp[255];

	ptrTemp = (char*)parPtr;
	printf("Starting thread : %s \n", ptrTemp);

	bfrTemp[0] = 0;

	while(1)
	{
		usleep(1000);

#ifdef DEBUG_SERIAL
		printf("SERIAL> begin to read\n");
		printf("length : %i \n",strlen(bfrTemp));
#endif
		if( ( wNBytes = read(mjamesSerialFile,bfrTemp , 255 ) ) > 0 )
		{
			bfrTemp[wNBytes] = 0;
			printf("%s",bfrTemp);
		}
	}
	return( parPtr );
}



/**
* jamesSetCallback : set the callback function to call when At received.
* @param parAtHandler : adress of the function to be called.
* @return none
*/
void jamesSetCallback( void (*parHandler)(char *) )
{
	jamesHandler = parHandler;
}
